# event-context
Event context for JS

Group related events. Similar to nodejs's domain module.

## Installation:
```
npm i -S event-context
```

## Usage

```js
import { createContext } from 'event-context';

const ctx = createContext();

ctx.run(() => {
  // run sync code
  doSthing();

  // run async code
  setTimeout(() => {
    doSthInFuture();
  }, 1000);
});

// at a later point in the app
// you can safely unbind timers created in this ctx
// every pending timers will not run
ctx.dispose();
```

## Plugins

`event-context` makes it easy to write plugins. Look at `src/plugins/jquery.js` for example.

To install plugins:

```
npm i -S event-context-plugin-jquery
```

and patch jquery before any binding. Upcoming versions will let you add existing events to a context

```js
import { createContext } from 'event-context';
import { patch } from 'event-context-plugins-jquery';

patch();

$(function () {
  // ... some code
  const ctx = createContext();
  ctx.run(() => {
    $('#awesome-button').click(function () {
      doSth();
    });
  });

  // later in your code
  ctx.dispose();

  // all bound event handlers will stop running.
  // events are removed from it DOM node.
});

```

## Roadmap

- [x] support node's `nextTick`
- [x] support node's `events`
- [ ] add contextStorage so you can save values to an event context and pass it to downstream functions (think of React's context)
- [ ] add child contexts (contexts that are created inside another context)
- [ ] add existing event emitters to a context
